@@ -1,14 +1,15 @@
-#' @importFrom stats model.matrix model.response delete.response model.frame terms
+#' @importFrom stats model.matrix model.response delete.response model.frame terms prcomp
 #' @importFrom stats model.weights logLik coef nobs vcov .getXlevels quantile runif approxfun sd
 #' @importFrom survival Surv is.Surv
-
+#' @importFrom splines ns bs
 
 
 #' @export
-pchreg <- function(formula, breaks, data, weights){
+pchreg <- function(formula, breaks, data, weights, splinex = NULL){
 
 	cl <- match.call()
 	mf <- match.call(expand.dots = FALSE)
+	mf$formula <- formula
 	m <- match(c("formula", "weights", "data"), names(mf), 0L)
 	mf <- mf[c(1L, m)]
 	mf$drop.unused.levels <- TRUE
@@ -25,7 +26,11 @@ pchreg <- function(formula, breaks, data, weights){
 	else{stop("only type = 'right' and type = 'counting' are supported")}
 	if(!(any(d == 1))){stop("all observation are censored")}
 	
-	x <- model.matrix(mt,mf)
+	x <- model.matrix(mt,mf); ax <- NULL
+	if(!is.null(splinex)){
+	  x <- build.splinex(x, splinex$method, splinex$df, splinex$degree, splinex$v)
+	  ax <- attributes(x)
+	 }
 	w <- model.weights(mf)
 	if(is.null(w)){w <- rep.int(1,n)}
 
@@ -35,6 +40,8 @@ pchreg <- function(formula, breaks, data, weights){
 
 	###
   
+	attr(mf, "rangex") <- fit$rangex
+	attr(mf, "splinex") <- ax
 	attr(mf, "u") <- fit$u
 	attr(mf, "y") <- fit$y
 	attr(mf, "type") <- type
@@ -47,7 +54,6 @@ pchreg <- function(formula, breaks, data, weights){
 	vn <- paste(rep.int(rownames(beta), r[2]), rep(1:r[2], each = r[1]), sep = "_")
 	dimnames(fit$covar) <- list(vn, vn)
 
-  
 	Hy <- predF.pch(fit, x, y)
 	Hz <- (if(type == "counting") predF.pch(fit,x,z)[,"Haz"] else 0)
 	logLik <- sum(d*log(Hy[,"haz"]), na.rm = TRUE) - sum(Hy[,"Haz"] - Hz)
@@ -56,7 +62,7 @@ pchreg <- function(formula, breaks, data, weights){
 	
 	fit <- list(call = cl, beta = beta, breaks = fit$breaks, 
 		covar = fit$covar, logLik = logLik, 
-		lambda = fit$lambda, Lambda = fit$Lambda, mf = mf)
+		lambda = fit$lambda, Lambda = fit$Lambda, mf = mf, x = x, conv.status = fit$conv.status)
 	class(fit) <- "pch"
 	fit
 }
@@ -83,10 +89,13 @@ predict.pch <- function(object, type = c("distr", "quantile", "sim"),
 	mt <- terms(mf)
 	xlev <- .getXlevels(mt, mf)
 	fittype <- attr(mf, "type")
+	splinex <- attr(mf, "splinex")
+	
 	obj <- list(beta = object$beta, breaks = object$breaks, y = attr(mf, "y"), 
-		lambda = object$lambda, Lambda = object$Lambda, u = attr(mf, "u"))
+		lambda = object$lambda, Lambda = object$Lambda, u = attr(mf, "u"), 
+		rangex = attr(mf, "rangex"))
 
-	if(missing(newdata)){
+	if((nodata <- missing(newdata))){
 		miss <- attr(mf, "na.action")
 		nomiss <- (if(is.null(miss)) 1:nrow(mf) else (1:(nrow(mf) + length(miss)))[-miss])
 	}
@@ -111,28 +120,32 @@ predict.pch <- function(object, type = c("distr", "quantile", "sim"),
 
 		miss <- attr(mf, "na.action")
 		nomiss <- (if(is.null(miss)) 1:nrow(mf) else (1:nrow(newdata))[-miss])
+    
+		x <- model.matrix(mt, mf)
+		if(!is.null(splinex)){
+		  x <- predict.splinex(x,splinex)
+		}
 	}
 
-	x <- model.matrix(mt, mf)
 	n <- length(miss) + length(nomiss)
 	if(type == "d"){
-		y <- cbind(model.response(mf))[,1 + (fittype == "counting")]
+		if(!nodata){y <- cbind(model.response(mf))[,1 + (fittype == "counting")]}
 		out <- matrix(, n, 4)
-		pred <- predF.pch(obj,x,y)
+		pred <- (if(nodata) predF.pch(obj) else predF.pch(obj,x,y))
 		out[nomiss,] <- pred
 		dimnames(out) <- list(1:n, colnames(pred))
 		return(as.data.frame(out))
 	}
   	else if(type == "q"){
 		out <- matrix(, n, length(p))
-		pred <- predQ.pch(obj,x,p)
+		pred <- (if(nodata) predQ.pch(obj, p = p) else predQ.pch(obj,x,p))
         	out[nomiss,] <- pred
 		dimnames(out) <- list(1:n, colnames(pred))
 		return(as.data.frame(out))
 	}
 	else{
 		tsim <- rep.int(NA,n)
-		tsim[nomiss] <- sim.pch(obj,x,method)
+		tsim[nomiss] <- (if(nodata) sim.pch(obj, method = method) else sim.pch(obj,x,method))
 		return(tsim)
 	}
 }
@@ -148,7 +161,7 @@ print.pch <- function(x, ...){
 summary.pch <- function(object, ...){
   out <- list(call = object$call, 
     n = attr(object$mf, "n"), n.events = attr(object$mf, "n.events"), 
-		n.free.par = attr(object$logLik, "df"), logLik = object$logLik)
+		n.free.par = attr(object$logLik, "df"), logLik = object$logLik, conv.status = object$conv.status)
   class(out) <- "summary.pch"
   out
 }
@@ -160,6 +173,7 @@ print.summary.pch <- function(x, ...){
   cat("n. of events: ", paste(deparse(round(x$n.events)), sep = " ", collapse = " "), "\n", sep = "")
   cat("n. of free parameters: ", paste(deparse(round(x$n.free.par)), sep = " ", collapse = " "), "\n", sep = "")
   cat("Log-likelihood: ", paste(deparse(round(as.numeric(x$logLik),1)), sep = " ", collapse = " "), "\n", sep = "")
+  cat("convergence status: ", paste(deparse(round(x$conv.status)), sep = " ", collapse = " "), "\n", sep = "")
   cat("Use predict() to obtain predictions from the fitted model")
   cat("\n")
 }
@@ -180,21 +194,28 @@ CumSum <- function(x){
 }
 
 
-makebreaks <- function(y, breaks){
+makebreaks <- function(y,d,x, breaks){
   
 	n <- length(y)
+	n1 <- sum(d)
+	q <- ncol(x)
 	r <- range(y)
-	if(missing(breaks)){breaks <- max(2, min(10, ceiling(n/50)))}
-  
+	y <- y[d == 1]
+	
+	if(missing(breaks)){breaks <- max(5, min(50, ceiling(n1/q/5)))}
+
 	if(length(breaks) > 1){
 		breaks <- sort(unique(breaks))
 		k <- length(breaks) - 1
 		if(r[1] < breaks[1] | r[2] > breaks[k + 1])
-			{stop("all y values must be within the breaks")}
+		  {stop("all y values must be within the breaks")}
+		breaks <- breaks[1:which(breaks >= r[2])[1]]
+		k <- length(breaks) - 1
 	}
 	else{
 		k <- breaks
 		breaks <- quantile(y, (0:k)/k)
+		breaks[1] <- r[1]; breaks[k + 1] <- r[2]
 		a <- duplicated(breaks)
 		if(any(a)){
 			for(j in which(a)){
@@ -215,7 +236,7 @@ makebreaks <- function(y, breaks){
 			k <- length(breaks) - 1
 		}
 	}
-	breaks[1] <- breaks[1] - (breaks[2] - breaks[1])/n
+	breaks[1] <- breaks[1] - (breaks[2] - breaks[1])/n1
 	names(breaks) <- NULL
 	list(breaks = breaks, k = k)	
 }
@@ -223,14 +244,14 @@ makebreaks <- function(y, breaks){
 
 
 
-pois.newton <- function(start, f, tol = 1e-5, maxit = 200, ...){
+pois.newton <- function(start, f, tol = 1e-5, maxit = 200, safeit = 0, ...){
 
 	f0 <- f(start, ..., deriv = 2)
 	g <- attr(f0, "gradient")
 	h <- attr(f0, "hessian")
 	conv <- FALSE
 	eps <- 1
-	alg <- "nr"
+	alg <- "gs"
 
 	for(i in 1:maxit){
 
@@ -238,16 +259,19 @@ pois.newton <- function(start, f, tol = 1e-5, maxit = 200, ...){
 
 		####
 		
-		H1 <- try(chol(h), silent = TRUE)
-		if(class(H1) != "try-error"){
-			if(alg == "gs"){alg <- "nr"; eps <- 1}
-			delta <- chol2inv(H1)%*%g
-		}
-		else{
-			if(alg == "nr"){alg <- "gs"; eps <- 1}
-			delta <- g
-		}
-
+	  	if(i > safeit){
+		  	H1 <- try(chol(h), silent = TRUE)
+		  	if(class(H1) != "try-error"){
+		  		if(alg == "gs"){alg <- "nr"; eps <- 1}
+		  		delta <- chol2inv(H1)%*%g
+		  	}
+		  	else{
+		  		if(alg == "nr"){alg <- "gs"; eps <- 1}
+		  		delta <- g
+		  	}
+	  	}
+		else{delta <- g}
+	  
 		####
 
 		f1 <- Inf
@@ -274,9 +298,9 @@ pois.newton <- function(start, f, tol = 1e-5, maxit = 200, ...){
 
 
 
-pois.loglik <- function(beta,d,x,w,off, deriv = 0){
+pois.loglik <- function(beta,d,x,w,off, zeror, deriv = 0){
 
-	log.lambda <- x%*%cbind(beta)
+	log.lambda <- x%*%cbind(beta) - zeror*1e+10
 	lambda <- exp(log.lambda)
 	a <- lambda*off
 	l <- sum(w*(d*log.lambda - a))
@@ -295,14 +319,20 @@ pois.loglik <- function(beta,d,x,w,off, deriv = 0){
 
 
 
-
+myapply <- function(x,fun){
+  out <- NULL
+  for(j in 1:ncol(x)){out <- rbind(out, fun(x[,j]))}
+  if(ncol(out) == 1){out <- c(out)}
+  out
+}
 
 poisfit <- function(d,x,w,off){
 
 	cn <- colnames(x)
 	if(!any(d != 0)){
-		sx <- apply(x,2,sd)
-		if(int <- (any((const <- (sx == 0))))){
+		sx <- myapply(x,sd)
+		const <- (if(nrow(x) > 1){sx == 0} else {colnames(x) == "(Intercept)"})
+		if(int <- any(const)){
 			beta <- rep(NA, ncol(x))
 			beta[const] <- -Inf
 			names(beta) <- cn
@@ -311,17 +341,37 @@ poisfit <- function(d,x,w,off){
 		else{stop("zero-risk can not be fitted unless an intercept is included")}
 	}
 
+	# Handling zero-risk regions
+
+	r <- myapply(x[d == 1,, drop = FALSE], range)
+	delta <- r[,2] - r[,1]
+	r[,1] <- r[,1] - 0.1*delta
+	r[,2] <- r[,2] + 0.1*delta
+	zeror <- rep.int(FALSE, length(d))
+	for(j in 1:ncol(x)){
+		out.l <- (x[,j] < r[j,1])
+		out.r <- (x[,j] > r[j,2])
+		if(mean(out.l) > mean(out.r)){outx <- out.l; r[j,2] <- Inf}
+		else{outx <- out.r; r[j,1] <- -Inf}
+		zeror <- (zeror | outx)
+	}
+	x <- x*(!zeror)
+
+  
+	# Scaling
+	
 	xx <- qr(x)
 	sel <- xx$pivot[1:xx$rank]
 	x <- x[,sel, drop = FALSE]
 	q <- ncol(x)
-	V <- list(x = x, off = off)
+	X <- list(x = x, off = off)
 
 	mx <- colMeans(x)
-	sx <- apply(x,2,sd)
+	sx <- myapply(x,sd)
 	M <- 10/max(off)
-	
-	if(int <- (any((const <- (sx == 0))))){
+
+	const <- (if(nrow(x) > 1){sx == 0} else {colnames(x) == "(Intercept)"})
+	if(int <- any(const)){
 		const <- which(const)
 		mx[const] <- 0
 		sx[const] <- x[1,const]
@@ -335,20 +385,43 @@ poisfit <- function(d,x,w,off){
 	vars <- which(sx > 0)
 	x <- scale(x, center = mx, scale = sx)
 
+	# Fit
+	
+	conv <- TRUE
 	beta0 <- rep.int(0,q)
-	fit <- pois.newton(beta0, pois.loglik, tol = 1e-5, maxit = 10*(1 + q), d = d, x = x, w = w, off = off)
+	if(int){beta0[const] <- max(-10, log(mean(d[!zeror])))}
+	safeit <- 0; fit.ok <- FALSE; count <- 0
+	while(!fit.ok){
+	  fit <- pois.newton(beta0, pois.loglik, tol = 1e-5, maxit = 10*(1 + q), safeit = safeit,
+	                     d = d, x = x, w = w, off = off, zeror = zeror)
+
+	  fit.ok <- all(abs(fit$gradient) < 1)
+	  count <- count + 1; safeit <- safeit + 2
+	  if(count == 20){conv <- FALSE; break}
+	}
 
 	beta <- fit$estimate
 	beta[vars] <- beta[vars]/sx[vars]
 	beta[const] <- beta[const] - sum(beta[vars]*mx[vars]) + log(M)
-	loglik <- pois.loglik(beta,d,V$x,w,V$off, deriv = 2)
-	h <- attr(loglik, "hessian")
+	loglik <- pois.loglik(beta,d,X$x,w,X$off, zeror, deriv = 2)
 
 	Beta <- rep(NA, length(xx$pivot))
 	Beta[sel] <- beta
 	names(Beta) <- cn
 
-	list(beta = Beta, vcov = chol2inv(chol(h)))
+	# covariance matrix
+
+	h <- attr(loglik, "hessian")
+	V <- 0*h
+	sel <- 1:ncol(h)
+	v <- try(chol2inv(chol(h)), silent = TRUE)
+	while(class(v) == "try-error"){
+	  qrh <- qr(h[sel,sel,drop = FALSE])
+	  sel <- sel[qrh$pivot[1:qrh$rank]]
+	  v <- try(chol2inv(chol(h[sel,sel, drop = FALSE])), silent = TRUE)
+	}
+	V[sel,sel] <- v
+	list(beta = Beta, vcov = V, r = r, converged = conv)
 }
 
 
@@ -363,7 +436,7 @@ pch.fit <- function(z,y,d,x,w,breaks){
 	if(missing(z)){z <- rep.int(-Inf,n)}
 	if(missing(d)){d <- rep.int(1,n)}
 	if(missing(w)){w <- rep.int(1,n)}
-	Breaks <- makebreaks(y,breaks)
+	Breaks <- makebreaks(y,d,x,breaks)
 
 	k <- Breaks$k
 	Breaks <- Breaks$breaks
@@ -376,8 +449,9 @@ pch.fit <- function(z,y,d,x,w,breaks){
 	end.y <- match(ycut, levels(ycut))
 	h <- Breaks[2:(k + 1)] - Breaks[1:k]
 
+	conv <- TRUE
 	A <- cbind(z,end.z,y,end.y,d,w,x)
-	beta <- NULL
+	beta <- NULL; rangex <- list()
 	V <- matrix(0,k*q, k*q)
 	for(j in 1:k){
 
@@ -395,40 +469,69 @@ pch.fit <- function(z,y,d,x,w,breaks){
 		r <- (yz_zj != 0)
 		dj <- dj[r]; yj_zj <- yz_zj[r]
 		xj <- xj[r,, drop = FALSE]; wj <- wj[r]
-
 		modj <- poisfit(dj, xj, wj, yj_zj)
 
 		beta <- cbind(beta, modj$beta)
+		rangex[[j]] <- modj$r
 		v <- matrix(0,q,q)
 		sel <- which(!is.na(modj$beta))
-		ind <- ((j - 1)*q + 1):(j*q)			
+		ind <- ((j - 1)*q + 1):(j*q)
 		v[sel,sel] <- modj$vcov
 		V[ind,ind] <- v
 
 		n <- nrow(A <- A[!uy,, drop = FALSE])
+		conv <- (conv & modj$converged)
 	}
 
 	beta[is.na(beta)] <- 0	
-	lambda <- exp(x%*%beta)
+	lambda <- cleanlambda(exp(x%*%beta),x,rangex)
 	Lambda <- CumSum(t(t(lambda)*h))
 	colnames(lambda) <- colnames(Lambda) <- 1:k
 
 	# y and corresponding interval
 	
-	y <- cbind(y, end.y, sort(y))
+	y <- cbind(y, end.y + 1, sort(y))
 	colnames(y) <- c("y","interval","sort.y")
 	attr(y, "tab.ycut") <- tab.ycut
 	attr(Breaks, "h") <- h; attr(Breaks, "k") <- k
-  
+	
 	# approxfun for quick prediction of the interval
-  
+	
 	br <- c(Breaks[1] - 1, Breaks)
 	u <- approxfun(br, c(1:k, k + 1, k + 1), rule = 2, method = "constant")  
+	
+	# check1: convergence
+	# check2: there should not be many survival = 1
+	
+	conv.status <- 0
+	if(!conv){conv.status <- 1; warning("the algorithm did not converge", call. = FALSE)}
+	else{
+	  surv <- exp(-cbind(0,Lambda)[cbind(1:nrow(Lambda), end.y)]) # approx survival
+	  if(mean(surv <= 1e-3) > 0.01){
+	    conv.status <- 2
+	    warning(
+	       "numerically zero survivals fitted: the solution may not be well-behaved
+	       (or the model is misspecified)", 
+	       call. = FALSE)
+	  }
+	}
 
 	list(beta = beta, lambda = lambda, Lambda = Lambda,
-		covar = V, breaks = Breaks, y = y, u = u)
+		covar = V, breaks = Breaks, y = y, u = u, rangex = rangex, conv.status = conv.status)
 }
 
+cleanlambda <- function(lambda,x,rangex){
+  for(h in 1:ncol(lambda)){
+    r <- rangex[[h]]
+    for(j in 1:ncol(x)){
+      e <- which(x[,j] < r[j,1] | x[,j] > r[j,2])
+      lambda[e,h] <- 0
+    }
+  }
+  pmin(lambda, 1e+6)
+}
+
+
 predF.pch <- function(obj,x,y){
 
   Breaks <- obj$breaks
@@ -436,13 +539,14 @@ predF.pch <- function(obj,x,y){
   h <- attr(Breaks, "h")
   beta <- obj$beta
   u <- obj$u
-  
+  rangex <- obj$rangex
+
   if(missing(x)){
     lambda <- obj$lambda
     Lambda <- obj$Lambda
   }
   else{
-    lambda <- exp(x%*%beta)
+    lambda <- cleanlambda(exp(x%*%beta), x, rangex)
     Lambda <- CumSum(t(t(lambda)*h))
   }
   
@@ -467,13 +571,14 @@ predQ.pch <- function(obj,x,p){ # p can be a vector (to simulate)
 	h <- attr(Breaks, "h")
 	k <- attr(Breaks, "k")
 	beta <- obj$beta
+	rangex <- obj$rangex
 
 	if(missing(x)){
 		lambda <- obj$lambda
 		Lambda <- obj$Lambda
 	}
 	else{
-		lambda <- exp(x%*%beta)
+		lambda <- cleanlambda(exp(x%*%beta), x, rangex)
 		Lambda <- CumSum(t(t(lambda)*h))
 	}
 	
@@ -513,6 +618,7 @@ sim.pch <- function(obj,x,method = c("q","s")){
 	h <- attr(Breaks, "h")
 	k <- attr(Breaks, "k")
 	beta <- obj$beta
+	rangex <- obj$rangex
 	y <- obj$y
 	tab.ycut <- attr(y, "tab.ycut")
 	y <- y[,"sort.y"]	
@@ -522,7 +628,7 @@ sim.pch <- function(obj,x,method = c("q","s")){
 		Lambda <- obj$Lambda
 	}
 	else{
-		lambda <- exp(x%*%beta)
+		lambda <- cleanlambda(exp(x%*%beta), x, rangex)
 		Lambda <- CumSum(t(t(lambda)*h))
 	}
 	
@@ -545,11 +651,107 @@ sim.pch <- function(obj,x,method = c("q","s")){
 		Lambda.out <- Lambda[u.out,k]
 		t[u.out] <- Breaks[k + 1] + (p.out - Lambda.out)/lambda.out
 	}
-
 	t
 }
 
 
+# splinex functions
 
+#' @export
+splinex <- function(method = c("ns", "bs"), df = 2, degree = 2, v = 0.98, ...){
+  if(is.na(match((method <- method[1]), c("ns", "bs")))){stop("invalid 'method'")}
+  if((df <- round(df)) < 1){stop("invalid 'df'")}
+  if((degree <- round(degree)) < 1){stop("invalid 'degree'")}
+  if(method == "bs" && df < degree){df <- degree; warning(paste("'df' was too small; have used", df))}
+  if(v <= 0 | v > 1){stop("0 < v <= 1 is required")}
+  list(method = method, df = df, degree = degree, v = v)
+}
+
+build.splinex <- function(x, method, df, degree, v){
+
+  m <- (if(method == "ns") function(x, df, deg) splines::ns(x, df = df) 
+        else function(x, df, deg) splines::bs(x, df = df, degree = deg))
+  x <- cbind("(Intercept)" = 1, as.matrix(x))
+  qx <- qr(x)
+  sel1 <- qx$pivot[1:qx$rank]
+  x <- x[,sel1, drop = FALSE]
+  cnx <- colnames(x)
+  
+  if(ncol(x) == 1){
+    attr(x, "onlyint") <- TRUE
+    return(x)
+  }
+  if(df == 1){
+    X <- x
+    df <- rep.int(1, ncol(X))
+    sel2 <- 1:ncol(X)
+    knots <- bknots <- NULL
+  }
+  else{
+    u <- myapply(x, function(a){length(unique(a))})
+    X <- cbind("(Intercept)" = rep.int(1, nrow(x)))
+    dfX <- 1; cnX <- "(Intercept)"
+    knots <- bknots <- list()
+    for(j in 2:ncol(x)){
+      dfj <- dfX[j] <- min(df, u[j] - 1)
+      xx <- (if(dfj > 1) m(x[,j], df = dfj, deg = degree) else x[,j])
+      X <- cbind(X, xx)
+      cnX <- c(cnX, paste(cnx[j],1:dfj, sep = "."))
+      knots[[j]] <- attr(xx, "knots")
+      bknots[[j]] <- attr(xx, "Boundary.knots")
+    }
+    colnames(X) <- cnX
+    df <- dfX
+    qX <- qr(X)
+    sel2 <- qX$pivot[1:qX$rank]
+    X <- X[,sel2, drop = FALSE]
+  }
+  if(v == 1){rot <- diag(ncol(X) - 1); center <- scale <- FALSE; sel3 <- 1:(ncol(X) - 1)}
+  else{
+    XX <- prcomp(X[,-1, drop = FALSE], center = TRUE, scale. = TRUE)
+    rot <- XX$rotation; center <- XX$center; scale <- XX$scale
+    vx <- cumsum(XX$sdev^2); vx <- vx/vx[length(vx)]
+    sel3 <- 1:(which(vx >= v)[1])
+    X <- cbind(X[,1,drop = FALSE],XX$x[,sel3])
+    colnames(X) <- c("(Intercept)", paste("pc", 1:(ncol(X) - 1), sep = ""))
+  }
+  
+  attr(X, "onlyint") <- FALSE
+  attr(X, "method") <- method[1]
+  attr(X, "sel1") <- sel1
+  attr(X, "df") <- df
+  attr(X, "degree") <- degree
+  attr(X, "knots") <- knots
+  attr(X, "bknots") <- bknots
+  attr(X, "sel2") <- sel2
+  attr(X, "center") <- center
+  attr(X, "scale") <- scale
+  attr(X, "rot") <- rot
+  attr(X, "sel3") <- sel3
+  X
+}
+
+predict.splinex <- function(x, a){
+  
+  if(a$onlyint){return(x)}
+
+  m <- (if(a$method == "ns") function(x, df, deg, kn, bkn) splines::ns(x, df = df, knots = kn, Boundary.knots = bkn) 
+        else function(x, df, deg, kn, bkn) splines::bs(x, df = df, degree = deg, knots = kn, Boundary.knots = bkn))
+  
+  x <- cbind(1,x)
+  x <- x[,a$sel1, drop = FALSE]
+  X <- 1
+  for(j in 2:ncol(x)){
+    xx <- (if(a$df[j] == 1) x[,j] else m(x[,j], df = a$df[j], deg = a$degree, 
+      kn = a$knots[[j]], bkn = a$bknots[[j]]))
+    X <- cbind(X, xx)
+  }
+  X <- X[,a$sel2, drop = FALSE]
+  X <- X[,-1,drop = FALSE]
+  X <- scale(X, center = a$center, scale = a$scale)
+  X <- X%*%a$rot
+  X <- X[,a$sel3, drop = FALSE]
+  cbind(1,X)
+}
 
 
